#import "AlarmWidgetPlugin.h"

@implementation AlarmWidgetPlugin

- (id)initWithWebView:(WebView *)webView {
    if (self = [super init]) {
		// Add observer: Shared NSWorkspace Notification
		[[[NSWorkspace sharedWorkspace] notificationCenter] addObserver:self
			selector:@selector(systemDidWake:)
			name:@"NSWorkspaceDidWakeNotification" object:nil];

		// Regist Growl
		[self setGrowlIcon:@"Icon.png"];
		[self growlRegist];
	}
	return self;
}

- (void)dealloc {
	// Remove observer from notification center
	[[[NSWorkspace sharedWorkspace] notificationCenter] removeObserver:self];

	[mySound release];
	[growlIconPath release];
	[webScriptObject release];
	[super dealloc];
}

- (void)windowScriptObjectAvailable:(WebScriptObject*) windowScriptObject {
	[windowScriptObject setValue:self forKey:@"AlarmWidgetPlugin"];

	if (webScriptObject)
		[webScriptObject release];
	webScriptObject = [windowScriptObject retain];
}

+ (NSString *)webScriptNameForSelector:(SEL)aSelector {
	if (aSelector == @selector(load:))
		return (@"load");
	if (aSelector == @selector(play:))
		return (@"play");
	if (aSelector == @selector(setGrowlIcon:))
		return (@"setGrowlIcon");
	if (aSelector == @selector(logMessage:))
		return (@"logMessage");
	return nil;
}

+ (BOOL)isSelectorExcludedFromWebScript:(SEL)aSelect {	
	if (aSelect == @selector(load:))
		return NO;
	if (aSelect == @selector(beep))
		return NO;
	if (aSelect == @selector(play:))
		return NO;
	if (aSelect == @selector(setGrowlIcon:))
		return NO;
	if (aSelect == @selector(stop))
		return NO;
	if (aSelect == @selector(logMessage:))
		return NO;
	return YES;
}

#pragma mark Implementation
- (void)load:(NSString *)path {
	if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
		if (mySound)
			[mySound release];
		mySound = [[NSSound alloc] initWithContentsOfFile:path byReference:NO];
		NSLog(@"Loaded sound file: %@", path);
	} else {
		NSLog (@"Could not find sound file: %@", path);
	}
}

- (void)play:(NSString *)times {
	int repeat = [times intValue];
	if ((repeat < 0) || (65536 <= repeat))
		repeat = 10;

	playCounter = 0;
    if ([mySound isPlaying])
		[mySound stop];
	[mySound setDelegate:self];
	playCounter = repeat;
	[self playLoop];
	[self growlNotify];
}

- (void)playLoop
{
	if (playCounter)
		[mySound play];
	playCounter--;
}

- (void)stop
{
	playCounter = 0;
    if ([mySound isPlaying])
		[mySound stop];
}

- (void)beep
{
	NSBeep();
}

- (void)sound:(NSSound *)sound didFinishPlaying:(BOOL)aBool
{
	[self playLoop];
}

// Get the name of this Widget
- (NSString *)nameOfWidget
{
	NSString		*widgetPath = [[[NSBundle bundleForClass:[self class]] bundlePath] stringByDeletingLastPathComponent];
	NSString		*infoPath = [widgetPath	stringByAppendingPathComponent:@"Info.plist"];
	NSDictionary	*infoDict = [NSDictionary dictionaryWithContentsOfFile:infoPath];
	NSString		*widgetName = nil;
	
	if (infoDict)
		widgetName = [infoDict objectForKey:@"CFBundleDisplayName"];
	if (widgetName)
		widgetName = [NSString stringWithFormat:@"%@ Widget", widgetName];
	else
		widgetName = @"Dashboard Widget";

	return widgetName;
}

// System waked up from the sleep mode
- (void)systemDidWake:(NSNotification*)aNotification
{
	#ifdef DEBUG
		NSLog(@"%@ is notified.", [aNotification name]);
	#endif
	[webScriptObject callWebScriptMethod:@"systemDidWake" withArguments:nil];
}

#pragma mark Growl
- (void)growlRegist
{
	if (![self checkIfRunningGrowl])
		return;

	NSAppleScript			*appleScript;
	NSString				*asSource;
	NSAppleEventDescriptor	*aeDescriptor;
	NSDictionary			*asErrDic = nil;
	NSString				*widgetName = [self nameOfWidget];

	asSource = [NSString stringWithFormat:@"%@\n%@\n%@\n%@\n%@%@%@\n%@\n%@",
		@"try",
		@"  tell application \"GrowlHelperApp\"",
		@"    set the allNotificationsList to {\"Timer Alarm\"}",
		@"    set the enabledNotificationsList to {\"Timer Alarm\"}",
		@"    register as application \"", widgetName, @"\" all notifications allNotificationsList default notifications enabledNotificationsList icon of application \"DashboardClient\"",
		@"  end tell",
		@"end try"];
	appleScript = [[NSAppleScript alloc] initWithSource:asSource];
	aeDescriptor = [appleScript executeAndReturnError : &asErrDic]; 
	#ifdef DEBUG
		NSLog (@"AppleEventDescriptor: %@", [aeDescriptor stringValue]);
	#endif
	[appleScript release];
}

- (void)growlNotify
{
	if (![self checkIfRunningGrowl])
		return;

	NSAppleScript			*appleScript;
	NSString				*asSource;
	NSAppleEventDescriptor	*aeDescriptor;
	NSDictionary			*asErrDic = nil;
//	NSString	*widgetPath = [[[NSBundle bundleForClass:[self class]] bundlePath]
//									stringByDeletingLastPathComponent];
//	NSString	*iconPath = [widgetPath	stringByAppendingPathComponent:@"Icon.png"];
	NSString	*widgetName = [self nameOfWidget];

	#ifdef DEBUG
//		NSLog(@"iconPath:%@", iconPath);
		NSLog(@"widgetName:%@", widgetName);
	#endif
	asSource = [NSString stringWithFormat:@"%@\n%@%@%@\n%@\n%@%@%@\n%@\n%@",
		@"try",
		@"  set the iconPath to \"file://", [self growlIconPath], @"\"",
		@"  tell application \"GrowlHelperApp\"",
		@"    notify with name \"Timer Alarm\" title \"Timer Alarm\" description \"It's Time!\" application name \"", widgetName, @"\" image from location iconPath",
		@"  end tell",
		@"end try"];
	appleScript = [[NSAppleScript alloc] initWithSource:asSource];
	aeDescriptor = [appleScript executeAndReturnError : &asErrDic]; 
	#ifdef DEBUG
		NSLog (@"AppleEventDescriptor: %@", [aeDescriptor stringValue]);
	#endif
	[appleScript release];
}

// Return YES if Growl is running
- (BOOL)checkIfRunningGrowl
{
	NSAppleScript			*appleScript;
	NSString				*asSource;
	NSAppleEventDescriptor	*aeDescriptor;
	NSDictionary			*asErrDic = nil;
	BOOL					flag = NO;

	asSource = [NSString stringWithFormat:@"%@\n%@\n%@\n%@\n",
		@"tell application \"System Events\"",
		@"  set isRunning to count of (every process whose name is \"GrowlHelperApp\") > 0",
		@"end tell",
		@"return isRunning"];
	appleScript = [[NSAppleScript alloc] initWithSource:asSource];
	aeDescriptor = [appleScript executeAndReturnError : &asErrDic]; 
	flag = [aeDescriptor booleanValue];
	[appleScript release];

	return flag;
}

// Set the Icon for Growl
- (BOOL)setGrowlIcon:(NSString *)iconFile {
	NSString	*widgetPath = [[[NSBundle bundleForClass:[self class]] bundlePath] stringByDeletingLastPathComponent];
	NSString	*iconPath;
	NSFileManager	*fileMan = [NSFileManager defaultManager];
	
	iconPath = [widgetPath stringByAppendingPathComponent:iconFile];

	if (![fileMan fileExistsAtPath:iconPath]) {
		iconPath = [widgetPath stringByAppendingPathComponent:@"Icon.png"];
		[self setGrowlIconPath:iconPath];
		return NO;
	}

	[self setGrowlIconPath:iconPath];
	return YES;
}

#pragma mark Accessor methods
- (NSString *)growlIconPath
{
	return growlIconPath;
}

- (void)setGrowlIconPath:(NSString *)newGrowlIconPath{
	[growlIconPath autorelease];
	growlIconPath = [newGrowlIconPath retain];
}

@end
